﻿<?php
include dirname(dirname(__DIR__)) . '/registrars/Manage_Domain/vendor/autoload.php';

use MDadd\ApiClient;
use MDadd\Balance;
use MDadd\Entity\Manager;
use MDadd\NicHandle;
use MDadd\OutPutUi;
use MDadd\Register;
use MDadd\SetPrice;
use MDreg\traits\checker;
use WHMCS\Database\Capsule as DB;
use WHMCS\ClientArea;

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

if (!defined("DS")) {
    define('DS', DIRECTORY_SEPARATOR);
}

function Manage_Domain_config()
{
    $configarray = array(
        "name" => "Manage Domain",
        "description" => "Manage domain Addons",
        "version" => "3.0.0",
        "author" => "Great world Lovers",
    );
    return $configarray;
}

function Manage_Domain_activate()
{
    try {
        if (!DB::schema()->hasTable('mod_MD_transactions')) {
            DB::schema()->create(
                'mod_MD_transactions',
                function ($table) {
                    $table->increments('id');
                    $table->string('amount');
                    $table->string('type');
                    $table->integer('userid');
                    $table->integer('invoiceid');
                    $table->string('description');
                    $table->string('status');
                    $table->timestamps();
                }
            );
        }
        if (!DB::schema()->hasTable('mod_md_nic_handles')) {
            DB::schema()->create(
                'mod_md_nic_handles',
                function ($table) {
                    $table->increments('id');
                    $table->string('nicHandle');
                    $table->date('validUntil');
                }
            );
        }
        if (!DB::schema()->hasTable('mod_MD_configs')) {
            DB::schema()->create(
                'mod_MD_configs',
                function ($table) {
                    $table->increments('id');
                    $table->string('key');
                    $table->string('value');
                }
            );
        }
        if (!DB::schema()->hasTable('mod_MD_tlds')) {
            DB::schema()->create(
                'mod_MD_tlds',
                function ($table) {
                    $table->increments('id');
                    $table->string('tld');
                    $table->integer('systemid');
                    $table->integer('lastupdate');
                }
            );
        }

        if (!DB::schema()->hasTable('mod_md_temp_price')) {
            DB::schema()->create(
                'mod_md_temp_price',
                function ($table) {
                    $table->increments('id');
                    $table->string('tld');
                    $table->text('price');
                    $table->timestamp('created_at');
                }
            );
        }
        if (!DB::schema()->hasTable('mod_md_private_nameservers')) {
            DB::schema()->create(
                'mod_md_private_nameservers',
                function ($table) {
                    $table->increments('id');
                    $table->integer('domain_id');
                    $table->text('ip', 50);
                    $table->text('nameserver');
                    $table->timestamp('sync');
                }
            );
        }

        $defaultCurrency = DB::table("tblcurrencies")->where("default", 1)->first();
        $data = [
            [
                "key" => "convertrate",
                "value" => "10",
            ],
            [
                "key" => "extrapercent",
                "value" => "1",
            ],
            [
                "key" => "defaultcurrency",
                "value" => $defaultCurrency->id,
            ],
        ];
        DB::table("mod_MD_configs")->insert($data);
    } catch (\Exception $e) {
        return array('status' => 'error', 'description' => $e->getMessage());
    }
    return array('status' => 'success', 'description' => 'Manage_domain active successfully');
}


function Manage_Domain_upgrade($vars)
{
    DB::schema()->table('tbldomains', function ($table) {
        if (!DB::schema()->hasColumn('tbldomains', 'MD_id')) {
            $table->integer('MD_id')->nullable();
        }
    });

    Manage_Domain_activate();
}

function Manage_Domain_deactivate()
{
    try {
        DB::schema()->dropIfExists('mod_MD_transactions');
        DB::schema()->dropIfExists('mod_MD_configs');
        DB::schema()->dropIfExists('mod_MD_tlds');
        DB::schema()->dropIfExists('mod_md_temp_price');
        DB::schema()->dropIfExists('mod_md_private_nameservers');
        DB::schema()->dropIfExists('mod_md_nic_handles');
    } catch (Exception $exception) {
        return array('status' => 'error', 'description' => $exception->getMessage());
    }
    return array('status' => 'success', 'description' => 'Manage_domain deactive successfully');
}

function Manage_Domain_clientarea($vars)
{
    $bg_colors = ['bg-secondary', 'bg-secondary', 'bg-dark', 'bg-info', 'bg-primary', 'bg-warning'];
    shuffle($bg_colors);
    if ($_REQUEST['action'] === 'check_available') {

        $captcha = new WHMCS\Utility\Captcha();


        check_token('WHMCS.default', $_REQUEST['token']);
        $error = false;
        $alreadyComplete = WHMCS\Session::get("CaptchaComplete");
        $counter = WHMCS\Session::get("CheckCounter");
        if ($counter < 20) {
            WHMCS\Session::set("CaptchaComplete", $counter++);
        } else {
            WHMCS\Session::set("CaptchaComplete", 0);
            WHMCS\Session::set("CheckCounter", false);
        }
        if (!$alreadyComplete) {

            $captcha = new WHMCS\Utility\Captcha();
            $validate = new WHMCS\Validate();
            $captcha->validateAppropriateCaptcha(WHMCS\Utility\Captcha::FORM_DOMAIN_CHECKER, $validate);
            if ($validate->hasErrors()) {
                WHMCS\Session::set("captchaData", array("invalidCaptcha" => true, "invalidCaptchaError" => $validate->getErrors()[0]));
                WHMCS\Session::set("CaptchaComplete", false);
            } else {
                WHMCS\Session::set("captchaData", array("invalidCaptcha" => false, "invalidCaptchaError" => false));
                WHMCS\Session::set("CaptchaComplete", true);
            }
        }

        if (!checker::getParams()) {
            $smarty = new Smarty;
            $smarty->assign('responseError', 'Confige information is not available or module not activated!');
            $smarty->template_dir = __DIR__ . '/templates';
            $smarty = $smarty->fetch('errors.tpl');
            die($smarty);
        }

        $domains = new WHMCS\Domains();
        $domainparts = $domains->splitAndCleanDomainInput($_REQUEST['domain']);

        $isValid = $domains->checkDomainisValid($domainparts);
        if (!$isValid) {
            $smarty = new Smarty;
            $smarty->assign('responseError', 'Input domain not valid!');
            $smarty->template_dir = __DIR__ . '/templates';
            $smarty = $smarty->fetch('errors.tpl');
            die($smarty);
        }

        $paramList = checker::$paramsList;
        $paramList['sld'] = $domainparts["sld"];
        $paramList['tlds'][] = $domainparts["tld"];

        $setPrice = new SetPrice;
        $suported_tlds = SetPrice::getTldList();

        if (!in_array($domainparts["tld"], $suported_tlds, true)) {
            $smarty = new Smarty;
            $smarty->assign('responseError', 'This tld is not supported!');
            $smarty->template_dir = __DIR__ . '/templates';
            $smarty = $smarty->fetch('errors.tpl');
            die($smarty);
        }

        $result = Manage_Domain_CheckAvailability($paramList);

        $result = $result->toArray();

        $smarty = new Smarty;
        $smarty->assign('response', $result);
        $smarty->assign('customFields', register::needsCustomFields($domainparts["tld"]));
        $smarty->assign('transferPricing', $setPrice->transferValue($domainparts["tld"]));
        $smarty->assign('MD5domain', $_REQUEST['MD5domain']);
        $smarty->assign('defaultns1', $_REQUEST['defaultns1']);
        $smarty->assign('defaultns2', $_REQUEST['defaultns2']);
        $smarty->assign('defualtNicHandle', $_REQUEST['defaultNicHandle']);
        $smarty->template_dir = __DIR__ . '/templates';
        $smarty = $smarty->fetch('checkAvailableResultCards.tpl');

        die($smarty);
    }

    if ($_REQUEST['action'] == 'register_domain' || $_REQUEST['action'] == 'transfer_domain') {

        if (!SetPrice::isLogin()) {
            $smarty = new Smarty;
            $smarty->assign('responseError', '\Please log in!');
            $smarty->template_dir = __DIR__ . '/templates';
            $smarty = $smarty->fetch('errors.tpl');
            die($smarty);
        }

        $domains = new WHMCS\Domains();
        $domainparts = $domains->splitAndCleanDomainInput($_REQUEST['domain']);

        if (substr($domainparts["tld"], -3) == '.ir') {

            $nicHandle = new NicHandle($_REQUEST['IRNIC_EMAIL_OR_HANDLE']);

            if (!$nicHandle->isValid()) {
                // ob_clean();
                http_response_code(400);
                $smarty = new Smarty;
                $smarty->assign('responseError', 'Nic handle not valid!');
                $smarty->template_dir = __DIR__ . '/templates';
                $smarty->assign('bg', $bg_colors[0]);
                $smarty = $smarty->fetch('errors.tpl');
                echo ($smarty);
                die;
            }
        }

        $register = new register($domainparts["tld"], $domainparts["sld"]);
        $exists = register::isAlreadypending($domainparts["tld"] . $domainparts["sld"]);
        if (!$exists) {
        }
        if ($_REQUEST['action'] == 'register_domain') {

            $action_result = $register->addRegisterOrder($_POST);
        } elseif ($_REQUEST['action'] == 'transfer_domain') {
            if ($_POST['year'] > 1 && substr($domainparts["tld"], -3) == '.ir') {

                WHMCS\Session::set("transferMoreThan1", true);
            }

            $action_result = $register->addTranferOrder($_POST);
        }

        $smarty = new Smarty;
        $smarty->assign('result', $action_result);
        $smarty->template_dir = __DIR__ . '/templates';
        $smarty = $smarty->fetch('successDataCard.tpl');
        die($smarty);
    }

    if ($_REQUEST['action'] == 'check_balance') {

        $result = Balance::getAPIuser();
        $currentUser = new \WHMCS\Authentication\CurrentUser;
        if ($result && $currentUser->isAuthenticatedUser()) {
            try {
                $fulBalance = $result['users'][0]['balance'];

                $credit = Balance::getCredit();

                return array(
                    'pagetitle' => 'Balance',
                    'breadcrumb' => array('index.php?m=Manage_Domain' => 'balance'),
                    'templatefile' => 'checkBalance',
                    'requirelogin' => true,
                    // 'forcessl' => false,
                    'vars' => array(
                        // "lang" => $vars['_lang'],
                        "lang" => [],
                        'credit' => $credit,
                        'Balance' => $fulBalance,
                        'prefix' => $result['currency']['iso'],
                        'suffix' => $result['currency']['symbolNative']
                    ),
                );
            } catch (Exception $e) {

                return array(
                    'pagetitle' => 'Balance',
                    'breadcrumb' => array('index.php?m=Manage_Domain' => 'balance'),
                    'templatefile' => 'errors',
                    'requirelogin' => true,
                    // 'forcessl' => false,
                    'vars' => array(
                        "responseError" => $e->getMessage()
                    ),
                );
            }
        }
        return array(
            'pagetitle' => 'Balance',
            'breadcrumb' => array('index.php?m=Manage_Domain' => $vars['_lang']["domain"]),
            'templatefile' => 'errors',
            'requirelogin' => true,
            'forcessl' => false,
            'vars' => array(
                "responseError" =>  'there is a problem in API',
            ),
        );
    }

    if ($_REQUEST['action'] == 'add_balance') {

        $result = Balance::getAPIuser();

        $currentUser = new \WHMCS\Authentication\CurrentUser;
        if ($result && $currentUser->isAuthenticatedUser()) {
            try {
                $balance = new Balance;
                $balance = $balance->addBalance($_POST['amount'], $result['users'][0]['id']);
                if (!$balance['success']) {
                    throw new Exception($balance['msg']);
                }
                $smarty = new Smarty;
                $smarty->assign('Balance', $balance['amount']);
                $smarty->assign('bg', $bg_colors[0]);
                $smarty->template_dir = __DIR__ . '/templates';
                $smarty = $smarty->fetch('successfullcharge.tpl');
                die($smarty);
            } catch (Exception $e) {
                $smarty = new Smarty;
                $smarty->assign('responseError', $e->getMessage());
                $smarty->template_dir = __DIR__ . '/templates';
                $smarty = $smarty->fetch('errors.tpl');
                die($smarty);
            }
        }
    }

    $captcha = new WHMCS\Utility\Captcha();


    $suggestTLD = SetPrice::suggestTld();


    return array(
        'pagetitle' => 'Group Register',
        'templatefile' => 'testregister',
        // 'requirelogin' => false,
        // 'forcessl' => false,
        'vars' => array(
            'captcha' => $captcha,
            'suggestTLD' => $suggestTLD,

        ),
    );
}

function Manage_Domain_getBalance()
{

    $result = DB::table("tblregistrars")->where("registrar", "Manage_Domain")->get();
    return $result;
}

function Manage_Domain_DecryptPassword($password)
{
    $command = 'DecryptPassword';
    $postData = array(
        'password2' => $password,
    );
    $results = localAPI($command, $postData);
    return $results;
}

function Manage_Domain_applycredit($params)
{
    $command = 'CreateInvoice';
    $postData = array(
        'userid' => $_SESSION["uid"],
        'itemdescription1' => 'invoice for charge domain account',
        'itemamount1' => $params["amount"],
        'autoapplycredit' => '1',
    );
    $results = localAPI($command, $postData);
    if ($results["result"] == "success") {
        DB::table("mod_MD_transactions")->insert([
            'amount' => $params["amount"],
            'userid' => $params['client']['userid'],
            'type' => '+',
            'status' => 'payed',
            'invoiceid' => $results["invoiceid"],
            'description' => 'پرداخت برای فروش دامنه',
        ]);
        DB::table("tblinvoiceitems")->where('invoiceid', $results["invoiceid"])->update(['type' => 'DomainPanel']);
    }
    return $results;
}

function Manage_Domain_GetClientsDetails($id)
{
    $command = 'GetClientsDetails';
    $postData = array(
        'clientid' => $id,
        'stats' => true,
    );
    $results = localAPI($command, $postData);
    return $results;
}

function Manage_Domain_ChargeAccount($params)
{

    $minesAccount = Manage_Domain_applycredit($params);
    $api = new ApiClient();

    $result = $api->post()->call("chargeAccount", $params);

    if ($result["result"] == "success") {
        header('Location: index.php?m=Manage_Domain&Status=success');
        die;
    } else {
        $command = 'AddCredit';
        $postData = array(
            'clientid' => $_SESSION["uid"],
            'description' => 'refund price off domain invoic : ' . $minesAccount["invoiceid"],
            'amount' => $params['amount'],
        );
        try {
            DB::table("mod_MD_transactions")
                ->where('invoiceid', $minesAccount["invoiceid"])
                ->update([
                    'status' => 'refund',
                    'description' => 'بازگشت موجودی به حساب کاربر',
                ]);
        } catch (Exception $ex) {
            echo $ex->getMessage();
            die;
        }
        $results = localAPI($command, $postData);
        if ($results["result"] == "success") {
            header('Location: index.php?m=Manage_Domain&Status=error&balance=notrefund');
            die;
        }
    }
}

function Manage_Domain_output($vars)
{
    try {
        $versionManager = new Manager();
        if (isset($_REQUEST["update"])) {
            if ($_REQUEST['update'] == '1') {

                header("Location: addonmodules.php?module=Manage_Domain&update=2&result=" . $versionManager->update() . "");
            } else if ($_REQUEST['update'] == '2') {
                if ($_REQUEST['result'] != '1') {
                    $output = '<div class="alert alert-danger text-center">';
                    $output .= $_REQUEST['result'];
                } else {
                    $output = '<div class="alert alert-success text-center">';
                    $output .= "SuccessFully Updated &nbsp;";
                }
                $output .= '</div>';
                echo $output;
            }
        }

        $version = $vars['version'];
        $remoteVersion = $versionManager->getLatestVersion();

        if (version_compare($version, $remoteVersion)) {
            $output = '<div class="alert alert-info text-center">';
            $output .= "New update is available (Current version: $version, Latest version: $remoteVersion)&nbsp;";
            $output .= 'click <a href="addonmodules.php?module=Manage_Domain&update=1"><strong>here</strong></a> to update.';
            $output .= '</div>';
            echo $output;
        }

        echo (new OutPutUi())->output();
    } catch (Exception $e) {
        $output = '<div class="alert alert-danger text-center">';
        $output .= $e->getMessage();
        $output .= '</div>';
        echo $output;
    }
}
