<?php
require_once __DIR__ . '/vendor/autoload.php';

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

if (!defined('DS')) {
    define('DS', DIRECTORY_SEPARATOR);
} else {
    // TODO error
}

use WHMCS\Domains\DomainLookup\ResultsList;
use WHMCS\Domains\DomainLookup\SearchResult;
use MDreg\domain;
use MDreg\api;
use MDreg\manageNameserver;
use WHMCS\Domains\DomainLookup\Provider\WhmcsDomains\ApiClient;
use WHMCS\Carbon;
use WHMCS\Database\Capsule as DB;
use WHMCS\Domain\Registrar\Domain as DomainWhmcs;

define('MANAGE_DOMAIN_ERROR_MESSAGE', 'We have problem to process this action.Please contact to admin.');

/**
 * Define module related metadata
 *
 * Provide some module information including the display name and API Version to
 * determine the method of decoding the input values.
 *
 * @return array
 */
function Manage_Domain_MetaData()
{
    return array(
        'DisplayName' => 'Manage Domain Registration',
        'APIVersion' => '2.0.0',
    );
}

/**
 * Define registrar configuration options.
 *
 * The values you return here define what configuration options
 * we store for the module. These values are made available to
 * each module function.
 *
 * You can store an unlimited number of configuration settings.
 * The following field types are supported:
 *  * Text
 *  * Password
 *  * Yes/No Checkboxes
 *  * Dropdown Menus
 *  * Radio Buttons
 *  * Text Areas
 *
 * @return array
 */
function Manage_Domain_getConfigArray()
{
    return [
        // Friendly display name for the module
        'FriendlyName' => [
            'Type' => 'System',
            'Value' => 'Manage Domain Registration Module.',
        ],
        'ApiKey' => [
            'FriendlyName' => 'API key',
            'Type' => 'password',
            'Size' => '500',
            'Default' => '',
            'Description' => 'Enter secret Api Key',
        ],
    ];
}

/**
 * Check Domain Availability.
 *
 * Determine if a domain or group of domains are available for
 * registration or transfer.
 *
 * @param array $params common module parameters
 * @return \WHMCS\Domains\DomainLookup\ResultsList An ArrayObject based collection of \WHMCS\Domains\DomainLookup\SearchResult results
 * @throws Exception Upon domain availability check failure.
 *
 */
function Manage_Domain_CheckAvailability($params)
{
    try {


        $ResultsList = new ResultsList();

        foreach ($params['tlds'] as $tld) {
            $domain = new domain($params);
            $domain->validateCheckAvailable($params);
            if (!$domain->isValid) {
                throw new Exception($domain->message, 10);
            }

            $domain->api->get()->makeUrl('checkAvailable', null, [], ['domain' => $params['sld'] . $tld]); //->exe();
            $urls[$tld]   =  str_replace('//' . $params['sld'], '//' . $params['sld'] . str_replace('.', '', $tld), $domain->api->finalUrl);
        }

        // Initialize cURL multi handle
        $mh = curl_multi_init();

        // Array to hold individual cURL handles
        $curlHandles = [];

        // Initialize individual cURL handles and add them to the multi handle
        foreach ($urls as $tld => $url) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Accept: application/json',
                "Authorization: Bearer " . $domain->api->apiKey,
                'Content-Type: application/x-www-form-urlencoded'

            ));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

            curl_multi_add_handle($mh, $ch);
            $curlHandles[$tld] = $ch;
        }


        // Execute the multi handle
        $running = null;
        do {
            $status = curl_multi_exec($mh, $running);
            if ($running) {
                curl_multi_select($mh);
            }
        } while ($running && $status == CURLM_OK);

        // Collect the results
        $results = [];
        foreach ($curlHandles as $tld => $ch) {

            $results[$tld] = curl_multi_getcontent($ch);
            curl_multi_remove_handle($mh, $ch);
            curl_close($ch);
        }

        // Close the multi handle
        curl_multi_close($mh);

        foreach ($results as $tld => $value) {
            $output = Manage_Domain_parse($value);
            // Instantiate a new domain search result object
            $searchResult = new SearchResult($params['sld'], $tld);

            if (!$output['success']) {
                if ($output['msg'] == 'this domain is already in your panel. you can not transfer again.') {
                    $status = 'exists_in_panel';
                    $searchResult->setStatus($status);
                    $ResultsList->append($searchResult);
                    continue;
                }
            }


            if (!$output['success']) {
                if ($output['msg'] == 'Tld is not valid or not supported.') {
                    $status = SearchResult::STATUS_TLD_NOT_SUPPORTED;
                } else {
                    $status = SearchResult::STATUS_UNKNOWN;
                }
            } else {
                // Determine the appropriate status to return
                if ($output['result']['check']['domainInfo']['availability'] == true) {
                    $status = SearchResult::STATUS_NOT_REGISTERED;
                } elseif ($output['result']['check']['domainInfo']['availability'] != true) {

                    $status = SearchResult::STATUS_REGISTERED;
                } else {
                    $status = SearchResult::STATUS_RESERVED;
                }
            }
            $searchResult->setStatus($status);
            $default_registrar = DB::table('mod_MD_configs')->where('key', 'registrar')->first();

            // Return premium information if applicable

            if ($output['result']['check']['domainInfo']['premium']) {
                $currency = DB::table(domain::modConfig_tbl)->where('key', 'currency')->first()->value;
                $currency = DB::table('tblcurrencies')->where('id', $currency ?? 1)->first()->code;
                $searchResult->setPremiumDomain(true);
                $searchResult->setPremiumCostPricing(
                    array(
                        'register' => $output['result']['check']['prices'][$default_registrar->value]['premium']['register'][1],
                        'transfer' => $output['result']['check']['prices'][$default_registrar->value]['premium']['transfer'][1],
                        'renew' => $output['result']['check']['prices'][$default_registrar->value]['premium']['register'][1],
                        'CurrencyCode' => $currency,
                    )
                );
            }

            // Append to the search results list
            $ResultsList->append($searchResult);
        }

        return $ResultsList;
    } catch (Exception $e) {
        //! tip : can't return error in here and have to return instanse of WHMCS\Domains\DomainLookup\SearchResult
        logModuleCall('ManageDomainModule', 'ERROR : CheckAvailability', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
    }
}

function Manage_Domain_parse($result)
{
    if (strlen($result) == 0) {
        return   ['success' => false, 'msg' => 'problem in get response.'];
    }

    $result = json_decode($result, true);
    if (isset($result['msg']) && is_array($result['msg'])) {
        $errors = "";
        foreach ($result['msg'] as $field => $message) {
            $errors .= (string)  $field . " : " .  $message[0] .  " , ";
        }
        $result['msg'] = $errors;
    }
    if (isset($result['message']) && is_array($result['message'])) {
        $errors = "";
        foreach ($result['message'] as $field => $message) {
            $errors .= (string)  $field . " : " .  $message[0] .  " , ";
        }
        $result['msg'] = $errors;
    }

    if ($result['status'] == 'success') {
        return [
            'success' => true,
            'msg' => $result['msg'] ?? $result['message'],
            'result' => $result['data'] ?? []
        ];
    }

    return  [
        'success' => false,
        'msg' => $result['msg'] ?? $result['message'],
        'result' => $result['data'] ?? []
    ];
}

/**
 * Initiate domain transfer.
 *
 * Attempt to create a domain transfer request for a given domain.
 *
 * This is triggered when the following events occur:
 * * Payment received for a domain transfer order
 * * When a pending domain transfer order is accepted
 * * Upon manual request by an admin user
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_TransferDomain($params)
{
    try {
        $domain = new domain($params);

        $domain->validateSaveContactDetails($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }
        //first create contact then use contact id and sent to transferParams
        $domain->api->put()->params($domain->transferContactParams())->makeUrl('createContact')->exe();
        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        $domain->validateTransfer($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $domain->api->params($domain->transferParams($domain->api->response['result']['id']))->post()->makeUrl('transfer')->exe();
        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        $domainid = (new \WHMCS\Domain\Domain())->find($params['domainid']);
        $domainid->MD_id = $domain->api->response['result']['id'];
        $domainid->save();

        logModuleCall('ManageDomainModule', 'TransferDomain', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => true);
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : TransferDomain', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Register a domain.
 *
 * Attempt to register a domain with the domain registrar.
 *
 * This is triggered when the following events occur:
 * * Payment received for a domain registration order
 * * When a pending domain registration order is accepted
 * * Upon manual request by an admin user
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_RegisterDomain($params)
{
    try {
        $domain = new domain($params);

        $domain->validateSaveContactDetails($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }
        //first create contact then use contact id and sent to transferParams
        $domain->api->put()->params($domain->transferContactParams())->makeUrl('createContact')->exe();
        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        $domain->validateTransfer($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $domain->api->params($domain->registerParams($domain->api->response['result']['id']))->post()->makeUrl('register')->exe();
        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        $domainid = (new \WHMCS\Domain\Domain())->find($params['domainid']);
        $domainid->MD_id = $domain->api->response['result']['id'];
        $domainid->save();

        logModuleCall('ManageDomainModule', 'TransferDomain', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => true);
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : TransferDomain', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Renew a domain.
 *
 * Attempt to renew/extend a domain for a given number of years.
 *
 * This is triggered when the following events occur:
 * * Payment received for a domain renewal order
 * * When a pending domain renewal order is accepted
 * * Upon manual request by an admin user
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_RenewDomain($params)
{
    try {
        $domain = (new domain($params))->getDomainID();
        $domain->validateRenew($params);
        if (!$domain->isValid) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        $domain->api->post()->params($domain->renewParams())->makeUrl('sync', '/' . $domain->domainid . '/renew')->exe();

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        logModuleCall('ManageDomainModule', 'RenewDomain', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array(
            'success' => true,
            'result' => $domain->api->response['msg']
        );
    } catch (Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : RenewDomain', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Fetch current nameservers.
 *
 * This function should return an array of nameservers for a given domain.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_GetNameservers($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $details = $domain->getDomainOrderInfo($domain->domainid, 'nameServers');

        $nameservers = [];
        foreach ($details as $key => $ns) {
            ++$key;
            $nameservers["ns$key"] = empty($ns) ? 'please update nameserver ...' : $ns;
        }

        logModuleCall('ManageDomainModule', 'GetNameservers', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return  $nameservers;
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : GetNameservers', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Save nameserver changes.
 *
 * This function should submit a change of nameservers request to the
 * domain registrar.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_SaveNameservers($params)
{
    try {
 
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $domain->validateSaveNameservers($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $domain->api->put()->params($domain->saveNameserversParams())->makeUrl('sync', '/' . $domain->domainid . '/nameServer')->exe();

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        logModuleCall('ManageDomainModule', 'SaveNameservers', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => true);
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : SaveNameservers', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Get the current WHOIS Contact Information.
 *
 * Should return a multi-level array of the contacts and name/address
 * fields that be modified.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function manage_Domain_GetContactDetails($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $details = $domain->getDomainOrderInfo($domain->domainid);

        if (substr($params['domainname'], -3) == '.ir') {
            return
                array(
                    'contact' => array(
                        'register' => $details["details"]["contactIds"]['reg'],
                        'admin' => $details["details"]["contactIds"]['admin'],
                        'billing' => $details["details"]["contactIds"]['billing'],
                        'technical' => $details["details"]["contactIds"]['tech'],
                    )
                );
        }


        if (is_null($details["contacts"]['reg'])) {

            return   array(
                'contact' => array(
                    'First Name' => 'Do not set yet',
                    'Last Name' => 'Do not set yet',
                    'Company' => 'Do not set yet',
                    'EMail' => 'Do not set yet',
                    'Address' => 'Do not set yet',
                    'State' => 'Do not set yet',
                    'City' => 'Do not set yet',
                    'Country' => 'US',
                    'Phone' => 'Do not set yet',
                    'Phone Country Code' => 'Do not set yet',
                    'Zip Code' => 'Do not set yet',
                )
            );
        }

        if (is_array(($details['reg']))) {
            return  array(
                'contact' => array(
                    'First Name' => $details['reg']['firstName'],
                    'Last Name' => $details['reg']['lastName'],
                    'Company' => $details['reg']['company'],
                    'EMail' => $details['reg']['email'],
                    'Address' => $details['reg']['address'],
                    'State' => $details['reg']['state'],
                    'City' => $details['reg']['city'],
                    'Country' => $details['reg']['country'],
                    'Phone' => $details['reg']['phone'],
                    'Phone Country Code' => $details['reg']['phoneCountryCode'],
                    'Zip Code' => $details['reg']['zipCode'],
                )
            );
        }
        return
            array(
                'contact' => array(
                    'register' => $details["contacts"]['reg'],
                    'admin' => $details["contacts"]['admin'],
                    'billing' => $details["contacts"]['billing'],
                    'technical' => $details["contacts"]['tech'],
                )
            );
    } catch (\Exception $e) {



        logModuleCall('ManageDomainModule', 'ERROR : GetContactDetails', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Update the WHOIS Contact Information for a given domain.
 *
 * Called when a change of WHOIS Information is requested within WHMCS.
 * Receives an array matching the format provided via the `GetContactDetails`
 * method with the values from the users input.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function manage_Domain_SaveContactDetails($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $domain->validateSaveContactDetails($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }
        //first create contact then use contact id and change save contact params
        $domain->api->put()->params($domain->saveContactDetailsParams())->makeUrl('createContact')->exe();
        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        $domain->api->put()->params($domain->changeContactDetailsParams($domain->api->response['result']['id']))->makeUrl('sync', '/' . $domain->domainid . '/contact')->exe();

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        logModuleCall('ManageDomainModule', 'SaveContactDetails', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => true);
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : SaveContactDetails', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Get registrar lock status.
 *
 * Also known as Domain Lock or Transfer Lock status.
 *
 * @param array $params common module parameters
 *
 * @return string|array Lock status or error message
 *
 */
function Manage_Domain_GetRegistrarLock($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $details = $domain->getDomainOrderInfo($domain->domainid, 'transferLock');


        if (isset($details['error'])) {
            throw new Exception($details['error'], 10);
        }

        return $details ? 'locked' : 'unlocked';
    } catch (\Exception $e) {


        logModuleCall('ManageDomainModule', 'ERROR : GetRegistrarLock', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Set registrar lock status.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_SaveRegistrarLock($params)
{
    try {

        $domain = (new domain($params))->getDomainID()->checkStatus();
        $domain->validateSaveRegistrarLock($params);

        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $domain->api->get()->makeUrl('sync', '/' . $domain->domainid . '/lock/toggle')->exe();

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        logModuleCall('ManageDomainModule', 'SaveRegistrarLock', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => $domain->api->response['msg']);
    } catch (\Exception $e) {

        logModuleCall('ManageDomainModule', 'ERROR : SaveRegistrarLock', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Get DNS Records for DNS Host Record Management.
 *
 * @param array $params common module parameters
 *
 * @return array DNS Host Records
 *
 */
function Manage_Domain_GetDNS($params)
{
    //! didn't need
    return;
}

/**
 * Update DNS Host Records.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_SaveDNS($params)
{
    //! didn't need
    return;
}

/**
 * Enable/Disable ID Protection.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_IDProtectToggle($params)
{
    //! didn't need
    return;
    // try {
    //     $domain = new domain($params);
    //     $domain->validateIDProtectToggle($params);
    //     if (!$domain->isValid) {
    //         throw new Exception($domain->message, 10);
    //     }

    //     $domain->api->put()->params($domain->IDProtectToggleParams())->makeUrl('IDProtectToggle')->exe();
    //     if (!$domain->response['success']) {
    //         throw new Exception($domain->response['msg'], 10);
    //     }

    //     logModuleCall('ManageDomainModule', 'IDProtectToggle', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
    //     return array('success' => 'success');
    // } catch (\Exception $e) {
    //     logModuleCall('ManageDomainModule', 'ERROR : IDProtectToggle', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
    //     return ($e->getCode() == 10) ?
    //         array('error' => $e->getMessage()) :
    //         array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    // }
}

/**
 * Request EEP Code.
 *
 * Supports both displaying the EPP Code directly to a user or indicating
 * that the EPP Code will be emailed to the registrant.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 *
 */
function Manage_Domain_GetEPPCode($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $domain->validateGetEPPCode($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $domain->api->get()->makeUrl('sync', '/' . $domain->domainid. '/epp')->exe();

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        logModuleCall('ManageDomainModule', 'GetEPPCode', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));

        if (isset($domain->api->response['result']['epp_code']) && !is_null($domain->api->response['result']['epp_code'])) {
            // If EPP Code is returned, return it for display to the end user
            return array('eppcode' => $domain->api->response['result']['epp_code']);
        } else {
            // If EPP Code is not returned, it was sent by email, return success
            return array('success' => 'success');
        }
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : GetEPPCode', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Register a Nameserver.
 *
 * Adds a child nameserver for the given domain name.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_RegisterNameserver($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $domain->validateRegisterNameserver($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $domain->api->post()->params($domain->registerNameserverParams())->makeUrl('sync', '/' . $domain->domainid . '/nameServer/private')->exe();

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        Manage_Domain_update_private_nameserver_table($params, $domain->api->response);

        domain::$nameserverPayload = $domain->api;


        logModuleCall('ManageDomainModule', 'RegisterNameserver', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => 'success');
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : RegisterNameserver', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Modify a Nameserver.
 *
 * Modifies the IP of a child nameserver.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_ModifyNameserver($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $domain->validateModifyNameserver($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $private_name_server = DB::table('mod_md_private_nameservers')->where('nameserver', $params["nameserver"]);

        $domain->api->put()->params($domain->modifyNameserverParams())->makeUrl('sync', '/' . $domain->domainid . '/nameServer/private/' . $private_name_server->first()->id)->exe();

        domain::$nameserverPayload = $domain->api;

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg'], 10);
        }

        $private_name_server->delete();

        Manage_Domain_update_private_nameserver_table($params, $domain->api->response);

        logModuleCall('ManageDomainModule', 'ModifyNameserver', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => 'success');
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : ModifyNameserver', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Delete a Nameserver.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_DeleteNameserver($params)
{
    try {
        $domain = (new domain($params))->getDomainID()->checkStatus();
        $domain->validateDeleteNameserver($params);
        if (!$domain->isValid) {
            throw new Exception($domain->message, 10);
        }

        $private_name_servers = DB::table('mod_md_private_nameservers')->where('nameserver', $params["nameserver"]);

        foreach ($private_name_servers->get() as $pns) {
            $domain->api->delete()->makeUrl('sync', '/' . $domain->domainid . '/nameServer/private/' . $pns->id)->exe();
            domain::$nameserverPayload = $domain->api;
            if (!$domain->api->response['success']) {
                throw new Exception($domain->api->response['msg'], 10);
            }
        }

        $private_name_servers->delete();

        Manage_Domain_update_private_nameserver_table($params, $domain->api->response);

        logModuleCall('ManageDomainModule', 'DeleteNameserver', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array('success' => 'success');
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : DeleteNameserver', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Sync Domain Status & Expiration Date.
 *
 * Domain syncing is intended to ensure domain status and expiry date
 * changes made directly at the domain registrar are synced to WHMCS.
 * It is called periodically for a domain.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_Sync($params, $force = false)
{
    try {
         $domain = (new domain($params))->getDomainID()->checkStatus();
        if (!$force && !Manage_Domain_can_sync($domain->model)) {
            return;
        }

        $details = $domain->api->get()->makeUrl('sync', '/' . $domain->domainid . '/sync')->exe();
        if ((!$details['success'])) {
            throw new Exception($details['error'], 10);
        }

        $info = $domain->getDomainOrderInfo($domain->domainid);

        if (isset($_SESSION["adminid"]) && !is_null($_SESSION["adminid"])) {
            // update status and expire date in tbldomains
            $domainModel = (new \WHMCS\Domain\Domain())->find($params['domainid']);

            $domain_status = [
                'active' => 'Active',
                'pending' => 'Pending',
                'pendingTransfer' => 'Pending Transfer',
                'pendingTransferOut' => 'Transferred Away',
                'pendingRegister' => 'Pending Registration',
                'pendingDelete' => 'Redemption',
                'deleted' => 'Expired',
                'expired' => 'Grace',
                'cancelled' => 'Cancelled',
            ];

            $domainModel->status = $domain_status[$info['details']['Status']];
            $domainModel->expirydate = $info['expireDate'];
            $domainModel->save();
        }

        logModuleCall('ManageDomainModule', 'Sync', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return array(
            'expirydate' => $info['expireDate'], // Format: YYYY-MM-DD
            'active' => (bool)($info['details']['Status'] == 'active'), // Return true if the domain is active
            'expired' => (bool)($info['details']['Status'] == 'expired'), // Return true if the domain has expired
            'transferredAway' => (bool)($info['details']['Status'] == 'transferred'), // Return true if the domain is transferred out
        );
    } catch (\Exception $e) {
          return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Incoming Domain Transfer Sync.
 *
 * Check status of incoming domain transfers and notify end-user upon
 * completion. This function is called daily for incoming domains.
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_TransferSync($params)
{
    try {
        $domain = new domain($params);
        $details = $domain->domainDetails();
        if (isset($details['error'])) {
            throw new Exception($details['error'], 10);
        }

        logModuleCall('ManageDomainModule', 'TransferSync', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        if ($details['status'] == 'active') {
            return array(
                'completed' => true,
                'expirydate' => $details['expireDate'], // Format: YYYY-MM-DD
            );
        }
        if ($details['status'] == 'cancelled') {
            return array(
                'failed' => true,
                'reason' => $details['note'] ?? '', // Reason for the transfer failure if available
            );
        }
        // No status change, return empty array
        return array();
    } catch (\Exception $e) {
        logModuleCall('ManageDomainModule', 'ERROR : TransferSync', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

/**
 * Client Area Custom Button Array.
 *
 * Allows you to define additional actions your module supports.
 * In this example, we register a Push Domain action which triggers
 * the `Manage_Domain_push` function when invoked.
 *
 * @return array
 */
function Manage_Domain_ClientAreaCustomButtonArray()
{
    return array();
}

/**
 * admin Custom Button Array.
 *
 * Allows you to define additional actions your module supports.
 * In this example, we register a Push Domain action which triggers
 *
 * @return array
 */
function Manage_Domain_AdminCustomButtonArray()
{
    return array(
        'Sync Domain' => 'force_sync',
    );
}

/**
 * Client Area Allowed Functions.
 *
 * Only the functions defined within this function or the Client Area
 * Custom Button Array can be invoked by client level users.
 *
 * @return array
 */
function Manage_Domain_ClientAreaAllowedFunctions()
{
    return array(
        'ManagenameServers' => 'private_nameserver_Manage',
    );
}

/**
 * Example Custom Module Function: Push
 *
 * @param array $params common module parameters
 *
 * @return array
 *
 */
function Manage_Domain_force_sync($params)
{
    return Manage_Domain_Sync($params, true);
}

/**
 * check can sync the domain
 *
 * @param array $domain
 * @return bool
 */
function Manage_Domain_can_sync($domain)
{
     $expireDate = Carbon::parse($domain->expirydate);
    $updatedAt = Carbon::parse($domain->updated_at);
    $next90 = Carbon::now()->addDays(90);
    $next30 = Carbon::now()->addDays(30);
    return ($domain->status == 'active' && (($expireDate < $next90) || ($updatedAt > $next30))) || $domain->status != 'active';
}

function Manage_Domain_private_nameserver_database()
{
    if (!DB::schema()->hasTable(domain::NS_TABLE)) {
        DB::schema()->create(
            domain::NS_TABLE,
            function ($table) {
                $table->increments('id');
                $table->integer('domain_id');
                $table->integer('laravel_domain_id');
                $table->string('ip', '50');
                $table->string('nameserver');
                $table->timestamp('sync');
                $table->charset = 'utf8';
                $table->collation = 'utf8_general_ci';
            }
        );
    }
    return DB::table(domain::NS_TABLE);
}

function Manage_Domain_private_nameserver_Manage($params)
{
    try {
        $manager = new manageNameserver($params, $_POST, $_GET);
        $response = $manager->{$_GET['subaction']}()->render();
        // logModuleCall('ManageDomainModule', 'nameserver_Manage', (domain::$nameserverPayload->getParams() ?? 'any params'), domain::$nameserverPayload->apiResponseWithoutParse ?? 'any response', domain::$nameserverPayload->response ?? 'any parsed response', ['whmcsParams' => $params, 'postParams' => $_POST, 'getParams' =>  $_GET] ?? []);
        Manage_Domain_response(['success' => true, 'result' => $response, 'msg' => $manager->getMessage()], 200);
    } catch (Exception $e) {
        if (is_null(domain::$nameserverPayload)) {
            logModuleCall('ManageDomainModule', 'ERROR : nameserver_Manage', $e->getMessage(), 'any response',  'any parsed response', ['whmcsParams' => $params, 'postParams' => $_POST, 'getParams' =>  $_GET] ?? []);
        } else {
            logModuleCall('ManageDomainModule', 'ERROR : nameserver_Manage', (domain::$nameserverPayload->getParams() ?? 'any params'), domain::$nameserverPayload->apiResponseWithoutParse ?? 'any response', domain::$nameserverPayload->response ?? 'any parsed response', ['whmcsParams' => $params, 'postParams' => $_POST, 'getParams' =>  $_GET] ?? []);
        }
        if ($e->getCode() == 10) {
            Manage_Domain_response(['success' => false, 'msg' => $e->getMessage()], 400);
        }
        Manage_Domain_response(['success' => false, 'msg' => MANAGE_DOMAIN_ERROR_MESSAGE], 400);
    }
}

function Manage_Domain_response($result, $code)
{
    ob_clean();
    http_response_code($code);
    echo json_encode($result);
    die(ob_get_clean());
}


function Manage_Domain_GetDomainInformation($params)
{
    try {

        if (is_null(domain::$details)) {
            $domain = (new domain($params))->getDomainID()->checkStatus();
            $response = $domain->getDomainOrderInfo($domain->domainid);
        } else {
            $response = domain::$details;
        }
        foreach ($response['nameServers'] as $key => $ns) {
            ++$key;
            $nameservers["ns$key"] = empty($ns) ? 'please update nameserver ...' : $ns;
        }

        return (new DomainWhmcs)
            ->setDomain($response['domain'])
            ->setNameservers($nameservers)
            ->setTransferLock($response["transferLock"] ? 'locked' : 'unlocked')
            // ->setExpiryDate(Carbon::createFromFormat('Y-m-d', Carbon::parse($response["details"]["Dates"]['Expiration'])))
            ->setRestorable(false)
            ->setIdProtectionStatus(false)
            ->setDnsManagementStatus(false)
            ->setEmailForwardingStatus(false);
    } catch (\Throwable $e) {
        // logModuleCall('ManageDomainModule', 'ERROR : GetDomainInformation', $domain->api->getParams() ?? 'any params', $domain->api->apiResponseWithoutParse ?? 'any response', $domain->api->response ?? 'any parsed response', json_encode($params));
        return ($e->getCode() == 10) ?
            array('error' => $e->getMessage()) :
            array('error' => MANAGE_DOMAIN_ERROR_MESSAGE);
    }
}

function Manage_Domain_update_private_nameserver_table($params, $response)
{
    DB::table('mod_md_private_nameservers')->where('domain_id', $params['domainid'])->delete();

    $insert = [];
    foreach ($response['result'] as $key => $value) {
        $insert[] = [
            "id" => $value['id'],
            "nameserver" => $value['nameserver'],
            "domain_id" => $params['domainid'],
            "ip" => $value['ips'][0],
            "sync" => $value['synced_at'],
        ];
    }

    DB::table('mod_md_private_nameservers')->insert($insert);
}
