<?php

namespace MDadd;

use Exception;
use WHMCS\Carbon;
use WHMCS\Database\Capsule as DB;
use WHMCS\User\Client;
use MDadd\ApiClient;

class NicHandle extends Client
{
    protected $nicHandle;

    public function __construct($nicHandle)
    {
        $this->nicHandle = $nicHandle;
    }

    public function isValid()
    {
        $exists = $this->existsAndValid();

        if (!$exists) {
            if ($this->checkValid($this->nicHandle)['status'] == 'success') {
                $this->updateValid($this->nicHandle);
                return true;
            }
            return false;
        }
        return true;
    }

    /**
     * searchs databace for nicHandle
     * @return bool if nic handle exists and not expierd 
     */
    public function existsAndValid()
    {
        $validNic = DB::table('mod_md_nic_handles')->where([["nicHandle", '=', $this->nicHandle], ['validUntil', '>', Carbon::now()]])->exists();

        // $expierd = DB::table('mod_md_nic_handles')->where([["nicHandle", '=', $this->nicHandle], ['validUntil', '<', Carbon::now()->timestamp]])->get();
        if ($validNic) {
            return true;
        }
        // elseif ($expierd) {
        //     return 0;
        // }
        return false;

        // $checknicHndlePayload = 1;
        // if ($checknicHndlePayload) {
        // }
        // DB::table('mod_md_nic_handles')->where("nicHandle", $_REQUEST['defualtNicHandle'])->update([
        //     'validUntil' => Carbon::now()->addHours('2')->timestamp,
        // ]);
    }

    public static function updateValid($nicHandle)
    {
        DB::table('mod_md_nic_handles')->updateOrInsert(
            ['nicHandle' => $nicHandle],
            ['validUntil' => Carbon::now()->addDay()]
        );
    }

    /**
     * @return bool
     *
     * */
    public static function checkValid($nicHandle)
    {
        try {
            $pattern = '/^[a-z]{2}[0-9]{2,5}-irnic$/';

            if (!preg_match($pattern, $nicHandle) && !self::isEmail($nicHandle)) {
                throw new Exception("Invalid NicHandle.");
            } else {
                $api = new ApiClient;
                $api->get()->call('nic/verify-handle?handle=' . $nicHandle);

                if (!$api->status) {
                    throw new Exception($api->results['message']);
                }

                return ['status' => 'success', 'data' => $api->results['data']];
            }
        } catch (\Throwable $th) {
            return ['status' => 'error', 'message' => $th->getMessage()];
        }
    }

    public static function isEmail($nicHandle)
    {
        if (filter_var($nicHandle, FILTER_VALIDATE_EMAIL)) {
            return true;
        }
        return false;
    }

    public static function checkDomain($domain)
    {
        try {
            $api = new ApiClient;
            $api->get()->call('nic/check-domain?domainName=' . $domain);

            if (!$api->status) {
                throw new Exception('Set the Irnic ID "fa5780-irnic" as the ID of the agent or financial interface of the domain.');
            }

            return ['status' => 'success'];
        } catch (\Throwable $th) {
            return ['status' => 'error', 'message' => $th->getMessage()];
        }
    }
}
