<?php

namespace MDreg;

use Exception;

class apiResponse
{
    private $statusCode, $resp,$curl;
    public $result,$apiResponseWithoutParse;

    const OK = 200;
    const BadRequest = 400;
    const Unauthorized = 401;
    const PaymentRequired = 402;
    const Forbidden = 403;
    const NotFound = 404;
    const MethodNotAllowed = 405;
    const RequestTimeout = 408;
    const ExpectationFailed = 417;
    const MisdirectedRequest = 421;
    const FailedDependency = 424;
    const TooManyRequest = 429;
    const LoginTimeout = 440;
    const NoResponse = 444;
    const RetryWith = 449;
    const InvalidToken = 498;
    const TokenRequired = 499;
    const SERVERERROR = 500;
    const ServiceUnavailable = 503;
    const HTTPNotSupport = 505;



    public function __construct($curl)
    {
        $this->curl = $curl;
        $this->resp = curl_exec($curl);
        $this->statusCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $this->apiResponseWithoutParse = $this->resp;
        // MD_dd($this->resp ,$this->statusCode ,curl_error($curl));
        $this->parse();
        curl_close($curl);
    }

    public function getStatusCode()
    {
        return $this->statusCode;
    }

    public function getResponse()
    {
        return $this->resp;
    }

    public function parse()
    {
        if ($this->resp === false) {
            $message = curl_error($this->curl) ?? 'NO_MESSAGE';
            $code = curl_errno($this->curl) ?? 'NO_CODE';
            // return ['success' => false, 'msg' => "API_ERROR = code :  $code ,message : $message"];
            throw new Exception("API_ERROR = code :  $code , message : $message");
        }

        if (strlen($this->resp) == 0) {
            return  $this->result = ['success' => false, 'msg' => 'problem in get response.'];
        }

        $this->resp = json_decode($this->resp,true);
        if (isset($this->resp['msg']) && is_array($this->resp['msg'])) {
            $errors = "";
            foreach ($this->resp['msg'] as $field => $message) {
                $errors .= (string)  $field . " : " .  $message[0] .  " , ";
            }
            $this->resp['msg'] = $errors;
        }
        if (isset($this->resp['message']) && is_array($this->resp['message'])) {
            $errors = "";
            foreach ($this->resp['message'] as $field => $message) {
                $errors .= (string)  $field . " : " .  $message[0] .  " , ";
            }
            $this->resp['msg'] = $errors;
        }
         if ($this->statusCode == self::OK) {
            return  $this->result = [
                'success' => true,
                'msg' => $this->resp['msg'] ?? $this->resp['message'],
                'result' => $this->resp['data'] ?? []
            ];
        }

        return  $this->result = [
            'success' => false,
            'msg' => $this->resp['msg'] ?? $this->resp['message'],
            'result' => $this->resp['data'] ?? []
        ];
    }
}
