<?php

namespace MDreg;

use Exception;
use MDreg\app;
use WHMCS\Database\Capsule as DB;

class domain extends app
{
    public $model;
    public $domainid;
    public static $details = null;
    public static $nameserverPayload;

    const DOMAIN_TABLE = 'tbldomains';
    const NS_TABLE = 'mod_md_private_nameservers';
    const tempPrice_tbl = 'mod_md_temp_price';
    const modConfig_tbl = 'mod_MD_configs';
    const addonModules_tbl = 'tbladdonmodules';
    const domainPricing_tbl = 'tbldomainpricing';
    const currencies_tbl = 'tblcurrencies';
    const pricing_tbl = 'tblpricing';
    const domains_tbl = 'tbldomains';


    function __construct($params)
    {
         parent::__construct($params);
        $this->model = DB::table(self::domains_tbl)->find($params['domainid']);
    }

    public function registerParams($contact_id)
    {
        $default_registrar = DB::table('mod_MD_configs')->where('key', 'registrar')->first();

        $this->params = array(
            'domain' => strtolower(trim($this->WhmcsParams['domainname'])),
            'nameServers' => [
                strtolower(trim($this->WhmcsParams['ns1'])),
                strtolower(trim($this->WhmcsParams['ns2']))
            ],
            'term' => $this->WhmcsParams['regperiod'],
            'driver' => substr($this->WhmcsParams['tld'], -2) == 'ir' ? 'nic' : $default_registrar->value,
            'neededFields' => $this->WhmcsParams['additionalfields'],
            'contact' => $contact_id,
            "isPremium" => $this->WhmcsParams['premiumEnabled'],
            'isIDN' => $this->WhmcsParams["is_idn"],
            'IDN' => array(
                'IDNLanguage' => $this->WhmcsParams["idnLanguage"],
                'tldPunyCode' => $this->WhmcsParams["tld_punycode"],
                'sldPunyCode' => $this->WhmcsParams["sld_punycode"],
            ),
        );
        return $this->params;
    }

    public function transferParams($contact_id)
    {
        $default_registrar = DB::table('mod_MD_configs')->where('key', 'registrar')->first();

        $this->params = array(
            'eppCode' => $this->WhmcsParams['eppcode'],
            'term' => $this->WhmcsParams['regperiod'],
            'driver' => substr($this->WhmcsParams['tld'], -2) == 'ir' ? 'nic' : $default_registrar->value,
            'neededFields' => $this->WhmcsParams['additionalfields'],
            'contact' => $contact_id,
        );
        return $this->params;
    }
    public function transferContactParams() //https://releases.whmcs.com/v2/packages.json
    {
        $this->params =  array(
            'firstName' => $this->WhmcsParams["firstname"],
            'lastName' => $this->WhmcsParams["lastname"],
            'email' => $this->WhmcsParams["email"],
            'company' => $this->WhmcsParams["companyname"],
            'city' => $this->WhmcsParams["city"],
            'state' => $this->WhmcsParams["state"],
            'country' => strtoupper($this->WhmcsParams["countrycode"]),
            'address' => $this->WhmcsParams["address1"],
            'zipCode' => $this->WhmcsParams["postcode"],
            'phone' => $this->WhmcsParams["phonenumber"],
            'phoneCode' => $this->WhmcsParams["phonecc"],
        );
        return $this->params;
    }

    public function renewParams()
    {
        return $this->params = array(
            'term' => $this->WhmcsParams['regperiod'],
            // 'domain' => strtolower(trim($this->WhmcsParams['domainname'])),
            "isPremium" => $this->WhmcsParams['premiumEnabled'],
            "restore" => $this->WhmcsParams['status'] == 'Redemption',
            'isIDN' => $this->WhmcsParams["is_idn"],
            'IDN' => array(
                'IDNLanguage' => $this->WhmcsParams["idnLanguage"],
                'tldPunyCode' => $this->WhmcsParams["tld_punycode"],
                'sldPunyCode' => $this->WhmcsParams["sld_punycode"],
            ),
        );
    }


    public function SaveRegistrarLockParams() 
    {
        $this->params = array(
            'status' => ($this->WhmcsParams['lockenabled'] == 'locked') ? 1 : 0,
            'domain' => strtolower(trim($this->WhmcsParams['domainname']))
        );
        return $this->params;
    }


    public function checkAvailableParams()
    {
        $this->params = strtolower(trim($this->WhmcsParams['sld'] . $this->WhmcsParams['tlds'][0]));
        return $this->params;
    }

    public function domainDetailsParams()
    {
        $this->params = strtolower(trim($this->WhmcsParams['domainname']));
        return $this->params;
    }

    function domainDetails($detail = null)
    {
        if (is_null(self::$details) || ($this->WhmcsParams['domainname'] != self::$details['domain'])) {
            $this->validateDomainDetails($this->WhmcsParams);
            if (!$this->isValid) {
                return array('error' => $this->message);
            }
            $details = $this->api->get()->makeUrl('domainDetails', '/' . $this->domainDetailsParams())->exe();
            if (!$details['success']) {
                return array('error' => $this->api->response['msg']);
            }
            self::$details = $details['result'];
        }

        return !is_null($detail) ?  self::$details[$detail] : self::$details;
    }

    function saveNameserversParams()
    {
        $this->params = array(
            "nameservers" => [
                $this->WhmcsParams['ns1'],
                $this->WhmcsParams['ns2'],
                $this->WhmcsParams['ns3'],
                $this->WhmcsParams['ns4'],
            ]
        );

        return $this->params;
    }

    function getEPPCodeParams()
    {
        $this->params =  strtolower(trim($this->WhmcsParams['domainname']));
        return $this->params;
    }


    function saveContactDetailsParams()
    {
        $this->params = array(
            'firstName' => $this->WhmcsParams['contactdetails']['contact']['First Name'] ?? null,
            'lastName' => $this->WhmcsParams['contactdetails']['contact']['Last Name'] ?? null,
            'companyName' => $this->WhmcsParams['contactdetails']['contact']['Company'] ?? null,
            'email' => $this->WhmcsParams['contactdetails']['contact']['EMail'] ?? null,
            'address' => $this->WhmcsParams['contactdetails']['contact']['Address'] ?? null,
            'state' => $this->WhmcsParams['contactdetails']['contact']['State'] ?? null,
            'city' => $this->WhmcsParams['contactdetails']['contact']['City'] ?? null,
            'country' => $this->WhmcsParams['contactdetails']['contact']['Country'] ?? null,
            'phone' => (int)$this->WhmcsParams['contactdetails']['contact']['Phone'] ?? null,
            'phoneCode' => $this->WhmcsParams['contactdetails']['contact']['Phone Country Code'] ?? null,
            'zipCode' => (int)$this->WhmcsParams['contactdetails']['contact']['Zip Code'] ?? null,
        );

        return $this->params;
    }

    function changeContactDetailsParams($contact_id)
    {
        $this->params = array(
            'admin' => $contact_id,
            'reg' => $contact_id,
            'tech' => $contact_id,
            'billing' => $contact_id,
        );

        return $this->params;
    }

    function IDProtectToggleParams()
    {
        $this->params = array(
            'status' => (bool)(!$this->WhmcsParams['protectenable']),
            'domain' => strtolower(trim($this->WhmcsParams['domainname'])),
        );
        return $this->params;
    }

    function registerNameserverParams()
    {
        $this->params = array(
            'nameserver' => str_replace('.' . ($this->WhmcsParams['domainname']), '', ($this->WhmcsParams['nameserver'])),
            'ip' => [$this->WhmcsParams['ipaddress']],
        );

        return $this->params;
    }

    function modifyNameserverParams()
    {
        $this->params = array(
            'nameserver' => str_replace('.' . ($this->WhmcsParams['domainname']), '', ($this->WhmcsParams['nameserver'])),
            'ip' => [$this->WhmcsParams['newipaddress']],
        );

        return $this->params;
    }

    function deleteNameserverParams()
    {
        $this->params = array(
            'domain' => strtolower(trim($this->WhmcsParams['domainname'])),
            'hostName' => $this->WhmcsParams['nameserver'],
            'ip' => $this->WhmcsParams['ip'],
        );
        return $this->params;
    }

    function syncPrivateNSParams()
    {
        $this->params = array(
            'domain' => strtolower(trim($this->WhmcsParams['domainname'])),
        );
        return $this->params;
    }

    // function getNameServers($domain_id)
    // {
    //     $this->api->get()->makeUrl('sync', '/' . $domain_id . '/nameServer')->exe();

    //     if (!$this->api->response['success']) {
    //         throw new Exception($this->api->response['msg'], 10);
    //     }

    //     return $this->api->response['result']['nameServers'];
    // }

    function getContactDetails($domain_id)
    {
        $this->api->get()->makeUrl('sync', '/' . $domain_id . '/contact')->exe();

        if (!$this->api->response['success']) {
            throw new Exception($this->api->response['msg'], 10);
        }

        return $this->api->response['result'];
    }

    function getDomainOrderInfo($domain_id, $detail = null)
    {
        if (is_null(self::$details) || ($this->WhmcsParams['domainname'] != self::$details['domain'])) {
            $this->validateDomainDetails($this->WhmcsParams);

            if (!$this->isValid) {
                return array('error' => $this->message);
            }

            $details = $this->api->get()->makeUrl('getDomainOrderInfo', '/' . $domain_id)->exe();

            if (!$details['success']) {
                return array('error' => $this->api->response['msg']);
            }

            self::$details = $details['result'];
        }

        return !is_null($detail) ?  self::$details[$detail] : self::$details;
    }

    public function getDomainID()
    {
        $domainid = (new \WHMCS\Domain\Domain())->find($this->WhmcsParams['domainid']);

        DB::table('tbldomainsadditionalfields')->where('domainid', $this->WhmcsParams['domainid'])->whereIn('name', ['irnichandle', 'nichandle'])->update(['name' => 'nicHandle']);

        if (!$domainid->MD_id) {
            $domain = new domain($this->WhmcsParams);

            $domain->api->get()->makeUrl('getByDomainName', null, [], ['domainName' => $domainid->domain])->exe();

            if (!$domain->api->response['success']) {
                if($domain->api->statusCode == apiResponse::NotFound &&  $domainid->status == "Active" ){
                    $domainid->status = "cancelled";
                    $domainid->save();
                }
                throw new Exception($domain->api->response['msg'], 10);
            }

            $domainid->MD_id = $domain->api->response['result']['id'];
            $domainid->save();

        }

        $this->domainid =  $domainid->MD_id;
        return $this;
    }

    public function checkStatus(){
        if(in_array($this->model->status,['Expired','Cancelled','Pending'])){
            throw new \Exception('domain not register',10);
        }
        return $this;
    }
}
