<?php


namespace MDreg;


use WHMCS\Carbon;
use Exception;
use WHMCS\Database\Capsule as DB;

class manageNameserver
{
    private $viewPath;
    private $formData;
    public $urlData;
    private $message;
    private $params;
    private $requiredFields;
    private static $connection;
    const ACTIONS = ['save', 'modify', 'delete', 'refactor', 'sync'];

    public function __construct($params, $formData, $urlData)
    {
        $this->viewPath = dirname(dirname(__DIR__)) . '/templates/PrivateNameServerTable.tpl';
        require_once dirname(dirname(__DIR__)) . '/Manage_Domain.php';
        $this->formData = $formData;
        $this->urlData = $urlData;
        $this->params = $params;
    }

    public function sync()
    {
        $this->validateAddress();
        $domain = (new domain($this->params ))->getDomainID();
        $domain->api->get()->makeUrl('sync', '/' . $domain->domainid. '/nameServer/private/sync')->exe();

        if (!$domain->api->response['success']) {
            throw new Exception($domain->api->response['msg']);
        }

        domain::$nameserverPayload = $domain->api;

        $this->parsePayload(domain::$nameserverPayload);

        $this->setMessage("Your name server's Updated!");
        
        return $this;
    }

    public function save()
    {
        $this->saveAndDeleteParams();
        if (
            $this->DBConnection(domain::NS_TABLE)
            ->where('nameserver', $this->formData['nameserver'])
            ->where('ip', $this->formData['ip'])
            ->exists()
        ) {
            throw new Exception('Nameserver already exists.');
        }
        Manage_Domain_RegisterNameserver($this->params);
        $this->parsePayload(domain::$nameserverPayload);
        $this->setMessage("Your Child Nameserver was saved. if you see any different sync the list.");
        return $this;
    }

    public function modify()
    {
        $this->modifyParams();
        if (
            $this->DBConnection(domain::NS_TABLE)
            ->where('nameserver', $this->formData['nameserver'])
            ->where('ip', $this->formData['newip'])
            ->exists()
        ) {
            throw new Exception('Your Child Nameserver was updated. if you see any different sync the list.');
        }
        Manage_Domain_RegisterNameserver($this->params);
        $this->parsePayload(domain::$nameserverPayload);
        $this->setMessage("Your Child Nameserver ");
        return $this;
    }

    public function delete()
    {
        $this->saveAndDeleteParams();
        Manage_Domain_RegisterNameserver($this->params);
        $this->parsePayload(domain::$nameserverPayload);
        $this->setMessage("Your Child Nameserver was deleted. if you see any different sync the list.");
        return $this;
    }

    public function render()
    {
        $nameservers = $this->DBConnection(domain::NS_TABLE)
            ->where('domain_id', $this->params['domainid'])->get();
        $nsArrayData = array(
            'msg' => $this->getMessage(),
            'list' => is_8() ? $nameservers->toArray() : $nameservers,
        );
        $smarty = new \WHMCS\Smarty;
        $smarty->assign('nameServers', $nsArrayData);
        $view = $smarty->fetch($this->viewPath);
        return $view;
    }

    public function parsePayload($apiResponse)
    {
        $payload = $apiResponse->response;
        if (!$payload['success']) {
            if ($apiResponse->statusCode == 400) {
                throw new Exception($payload['msg']);
            }
            $this->DBManager(domain::NS_TABLE, $payload['result']);
            if ($apiResponse->statusCode != 404) {
                throw new Exception($payload['msg']);
            }
        } else {
            $this->DBManager(domain::NS_TABLE, $payload['result']);
        }
    }

    public function modifyParams()
    {
        $this->requiredFields = ['domain', 'nameserver', 'ip', 'newip'];
        $this->validate();
        foreach ($this->requiredFields as $field) {
            $this->params[$field] = $this->formData[$field];
        }
    }

    public function saveAndDeleteParams()
    {
        $this->requiredFields = ['domain', 'nameserver', 'ip'];
        $this->validate();
        foreach ($this->requiredFields as $field) {
            $this->params[$field] = $this->formData[$field];
        }
    }

    public function validate()
    {
        $this->validateAddress();
        $this->validateFormData();
        $this->validateInfos();
        return $this;
    }

    public function validateAddress()
    {
        if (!isset($this->urlData['subaction']) && !in_array($this->urlData['subaction'], self::ACTIONS)) {
            throw new Exception('url parameter in not valid, Please refresh the page and try again or contact to admin.', 10);
        }
        return $this;
    }

    public function validateFormData()
    {
        foreach ($this->formData as $key => $value) {
            $this->formData[$key] = trim($value);
        }
        foreach ($this->requiredFields as $value) {
            if (empty($this->formData[$value])) {
                throw new Exception('form parameter in not valid, Please refresh the page and try again or contact to admin.', 10);
            }
        }
        if (empty($this->formData['id'])) {
            throw new Exception('form parameter in not valid, Please refresh the page and try again or contact to adminid.', 10);
        }

        return $this;
    }

    public function validateInfos()
    {
        if (!$this->DBConnection(domain::DOMAIN_TABLE)->where('id', $this->formData['id'])->exists()) {
            throw new Exception('form parameter in not valid, Please refresh the page and try again or contact to admin.', 10);
        }
        return $this;
    }

    public function DBManager($table, $data)
    {
        $this->DBConnection($table);
        return $this->DBRefactor($data);
    }

    public function DBConnection($table)
    {
        if (isset(self::$connection) && (self::$connection->from == $table)) {
            return self::$connection;
        }
        return self::$connection = DB::table($table);
    }

    public function DBRefactor($data)
    {
        self::$connection->where('domain_id', $this->params['domainid'])->delete();
        foreach ($data as $nameserver) {
            foreach ($nameserver['ips'] as   $ip) {
                self::$connection->updateOrInsert([
                    'nameserver' => $nameserver['nameserver'],
                    'ip' => $ip,
                    'domain_id' => $this->params['domainid'],
                    'upstream_id' => $nameserver['id'],
                ], [
                    'sync' => Carbon::now(),
                ]);
            }
        }
        $this->setMessage("Your name server's list was updated successfully.");
        return true;
    }

    public function setFromData($formData)
    {
        $this->formData = $formData;
        return $this;
    }

    public function getParams()
    {
        return $this->params;
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function setMessage($message)
    {
        $this->message = $message;
    }
}
